ucx/cx/array_list.h

branch
newapi
changeset 253
087cc9216f28
parent 174
0358f1d9c506
--- a/ucx/cx/array_list.h	Sun Feb 11 15:44:33 2024 +0100
+++ b/ucx/cx/array_list.h	Sun Feb 11 22:06:23 2024 +0100
@@ -31,7 +31,6 @@
  * \details Also provides several low-level functions for custom array list implementations.
  * \author Mike Becker
  * \author Olaf Wintermann
- * \version 3.0
  * \copyright 2-Clause BSD License
  */
 
@@ -46,6 +45,11 @@
 #endif
 
 /**
+ * The maximum item size in an array list that fits into stack buffer when swapped.
+ */
+extern unsigned cx_array_swap_sbo_size;
+
+/**
  * Defines a reallocation mechanism for arrays.
  */
 struct cx_array_reallocator_s {
@@ -54,8 +58,9 @@
      *
      * Implementations are not required to free the original array.
      * This allows re-allocation of static memory by allocating heap memory
-     * and copying the array contents. The information in \p data can keep
-     * track of the state of the memory or other additional allocator info.
+     * and copying the array contents. The information in the custom fields of
+     * the referenced allocator can be used to track the state of the memory
+     * or to transport other additional data.
      *
      * @param array the array to reallocate
      * @param capacity the new capacity (number of elements)
@@ -89,12 +94,17 @@
 };
 
 /**
- * Return codes for cx_array_copy().
+ * A default stdlib-based array reallocator.
  */
-enum cx_array_copy_result {
-    CX_ARRAY_COPY_SUCCESS,
-    CX_ARRAY_COPY_REALLOC_NOT_SUPPORTED,
-    CX_ARRAY_COPY_REALLOC_FAILED,
+extern struct cx_array_reallocator_s *cx_array_default_reallocator;
+
+/**
+ * Return codes for array functions.
+ */
+enum cx_array_result {
+    CX_ARRAY_SUCCESS,
+    CX_ARRAY_REALLOC_NOT_SUPPORTED,
+    CX_ARRAY_REALLOC_FAILED,
 };
 
 /**
@@ -107,7 +117,7 @@
  * capacity is used.
  *
  * If the capacity is insufficient to hold the new data, a reallocation
  * attempt is made, unless the allocator is set to \c NULL, in which case
  * this function ultimately returns a failure.
  *
  * @param target the target array
@@ -122,7 +132,7 @@
  * if re-allocation shall not happen
  * @return zero on success, non-zero error code on failure
  */
-enum cx_array_copy_result cx_array_copy(
+enum cx_array_result cx_array_copy(
         void **target,
         size_t *size,
         size_t *capacity,
@@ -133,6 +143,28 @@
         struct cx_array_reallocator_s *reallocator
 ) __attribute__((__nonnull__(1, 2, 5)));
 
+/**
+ * Adds an element to an array with the possibility of allocating more space.
+ *
+ * The element \p elem is added to the end of the \p target array which containing
+ * \p size elements, already. The \p capacity must not be \c NULL and point a
+ * variable holding the current maximum number of elements the array can hold.
+ *
+ * If the capacity is insufficient to hold the new element, and the optional
+ * \p reallocator is not \c NULL, an attempt increase the \p capacity is made
+ * and the new capacity is written back.
+ *
+ * @param target the target array
+ * @param size a pointer to the size of the target array
+ * @param capacity a pointer to the target array's capacity - must not be \c NULL
+ * @param elem_size the size of one element
+ * @param elem the element to add
+ * @param reallocator the array re-allocator to use, or \c NULL
+ * if re-allocation shall not happen
+ * @return zero on success, non-zero error code on failure
+ */
+#define cx_array_add(target, size, capacity, elem_size, elem, reallocator) \
+    cx_array_copy((void**)(target), size, capacity, *(size), elem, elem_size, 1, reallocator)
 
 /**
  * Swaps two array elements.
@@ -153,12 +185,14 @@
  * Allocates an array list for storing elements with \p item_size bytes each.
  *
  * If \p item_size is CX_STORE_POINTERS, the created list will be created as if
- * cxListStorePointers() was called immediately after creation.
+ * cxListStorePointers() was called immediately after creation and the compare
+ * function will be automatically set to cx_cmp_ptr(), if none is given.
  *
  * @param allocator the allocator for allocating the list memory
  * (if \c NULL the cxDefaultAllocator will be used)
  * @param comparator the comparator for the elements
- * (if \c NULL sort and find functions will not work)
+ * (if \c NULL, and the list is not storing pointers, sort and find
+ * functions will not work)
  * @param item_size the size of each element in bytes
  * @param initial_capacity the initial number of elements the array can store
  * @return the created list
@@ -178,7 +212,8 @@
  * set it immediately after creation or use cxArrayListCreate().
  *
  * If \p item_size is CX_STORE_POINTERS, the created list will be created as if
- * cxListStorePointers() was called immediately after creation.
+ * cxListStorePointers() was called immediately after creation and the compare
+ * function will be automatically set to cx_cmp_ptr().
  *
  * @param item_size the size of each element in bytes
  * @param initial_capacity the initial number of elements the array can store
@@ -107,7 +117,7 @@
  *
  * If the capacity is insufficient to hold the new data, a reallocation
- * attempt is made, unless the allocator is set to \c NULL, in which case
+ * attempt is made, unless the \p reallocator is set to \c NULL, in which case
  * this function ultimately returns a failure.
  *

mercurial